import requests
import bibtexparser
from bs4 import BeautifulSoup
from multiprocessing.pool import ThreadPool
from functools import partial
import logging
logging.basicConfig(level=logging.DEBUG)
logging.getLogger("requests").setLevel(logging.WARNING)
logging.getLogger("urllib3").setLevel(logging.WARNING)
logging.getLogger("bibtexparser").setLevel(logging.WARNING)


def get_document_metadata(document_id):
    BASEURL = "https://air.unimi.it/references"
    r = requests.get(BASEURL, params={'format': 'bibtex', 'item_id': str(document_id)})
    return bibtexparser.loads(r.text).entries[0]


def get_documents_from_author(author_id):
    offset = 0
    BUNCH = 10

    # get = partial(get_documents_from_author_offset, author_id=author_id)

    def get(offset):
        return get_documents_from_author_offset(author_id, offset)

    while True:
        with ThreadPool(BUNCH) as pool:
            offset_bunch = []
            for b in range(BUNCH):
                offset_bunch.append(offset)
                offset += 20

            r = pool.map(get, offset_bunch)
            for rr in r:
                dobreak = False
                for rrr in rr:
                    yield rrr
                if not rr:
                    dobreak = True
            if dobreak:
                break


def get_documents_from_author_offset(author_id, offset):
    BASEURL = "https://air.unimi.it/browse?type=author&order=ASC&rpp=20&authority=%s&offset=%d"
    url = BASEURL % (author_id, offset)
    logging.debug("getting info from %s", url)
    r = requests.get(url)
    html = r.text
    soup = BeautifulSoup(html, 'lxml')

    form = soup.find("form", {'id': 'exportform'})
    if not form:
        return []

    entries = form.find_all("input", {'name': 'item_id'})
    return [entry['value'] for entry in entries]


g = get_documents_from_author('rp09852')
for gg in g:
    info = get_document_metadata(gg)
    if 'doi' not in info:
        logging.info("skipping %s", info)
        continue
    print(info['doi'])


import logging
logging.basicConfig(level=logging.INFO)
import re
import xmltodict

from common import get_html
import dump_keyword

from tkinter import *


def format_name(author):
    name = author[1]
    surname = author[0]
    names = name.split()
    initials = "".join(["%s." % n[0] for n in names])
    return "%s %s" % (initials, surname)


def format_name_italian(author):
    name = author[1]
    surname = author[0]
    return "%s, %s" % (surname, name)


def parse(html_inspire, default_institution):
    m = re.search(r"/([0-9]+)", html_inspire)
    if m is None:
        raise ValueError("not valid html")
    inspire_number = m.group(1)
    url = "http://inspirehep.net/record/{0}/export/xn".format(inspire_number)
    xml = get_html(url)
    doc = xmltodict.parse(xml)

    authors = get_authors(doc)

    print("\n" + "=" * 10 + " ALL AUTHORS " + "=" * 10)
    authors_list = ", ".join(map(format_name, authors))
    print(authors_list)

    print("\n found %d authors" % len(authors))
    milan_authors = [author for author in authors if (default_institution in " ".join(author[2]))]

    print("\n" + "=" * 10 + (" %s AUTHORS " % default_institution) + "=" * 10)
    milan_list = "\n".join(map(format_name_italian, milan_authors))
    print(milan_list)

    print("\n" + "=" * 10 + " TITLE " + "=" * 10)
    title = get_title(doc)
    print(title)

    print("\n" + "=" * 10 + " ABSTRACT " + "=" * 10)
    abstract = get_abstract(doc)
    print(abstract)

    print("\n===== KEYWORKDS ======\n")
    keys = dump_keyword.get_keys_from_html(get_html(html_inspire))
    print(keys)

    return authors_list, milan_list, title, abstract, keys


def get_abstract(xml_dict):
    return xml_dict['articles']['article']['front']['abstract']


def get_title(xml_dict):
    meta = xml_dict['articles']['article']['front']['article-meta']
    title = meta['title-group']['article-title']
    return title


def get_authors(xml_dict):
    authors = []

    meta = xml_dict['articles']['article']['front']['article-meta']
    contrib = meta['contrib-group']['contrib']
    for c in contrib:
        author_institutions = []
        try:
            aff = c["aff"]
            if len(aff) == 1:
                author_institutions.append(aff["institution"])
            else:
                for a in aff:
                    author_institutions.append(a["institution"])

        except KeyError:
            logging.warning("author %s %s has no institution, check manually" % (c["name"]["surname"], c["name"]["given-names"]))
            author_intitutions = ["unknown"]
                        

        authors.append((c["name"]["surname"],
                        c["name"]["given-names"],
                        author_institutions))
    return authors


class Application(Frame):
    def run(self):
        url = self.input_inspirehep.get()

        self.text_titles.delete("1.0", END)
        self.text_all_authors.delete("1.0", END)
        self.text_milan_authors.delete("1.0", END)
        self.text_abstract.delete("1.0", END)
        self.text_keywords.delete("1.0", END)

        authors_list, milan_list, title, abstract, keys = parse(url,
                                                               self.institution)
        
        self.text_titles.insert(INSERT, title)
        self.text_all_authors.insert(INSERT, authors_list)
        self.text_milan_authors.insert(INSERT, milan_list)
        self.text_abstract.insert(INSERT, abstract)
        self.text_keywords.insert(INSERT, keys)

    def say_hi(self):
        print("hi there, everyone!")

    def copy(self, widget):
        text = widget.get("1.0", END)
        self.clipboard_clear()
        self.clipboard_append(text)

    def createWidgets(self, url):
        self.label_input = Label(self, text="inspirehep url:")
        self.label_input.grid(row=0, sticky=W)
        self.input_inspirehep = Entry(self)
        self.input_inspirehep.configure(width=50)
        self.input_inspirehep.grid(row=0, column=1)
        self.input_inspirehep.insert(INSERT, url)
        self.button_run = Button(self, text="run", command=self.run)
        self.button_run.grid(row=0, column=2)

        self.text_titles = Text(self)
        self.text_titles.config(height=2)
        self.text_titles.grid(row=1, sticky=N, columnspan=2)
        self.copy_button_titles = Button(self, text="copy",
                                         command=lambda: self.copy(self.text_titles))
        self.copy_button_titles.grid(row=1, column=2)

        self.text_all_authors = Text(self)
        self.text_all_authors.config(height=10)
        self.text_all_authors.grid(row=2, sticky=N, columnspan=2)
        self.copy_button_all_authors = Button(self, text="copy",
                                              command=lambda: self.copy(self.text_all_authors))
        self.copy_button_all_authors.grid(row=2, column=2)

        self.text_milan_authors = Text(self)
        self.text_milan_authors.config(height=10)
        self.text_milan_authors.grid(row=3, sticky=N, columnspan=2)
        self.copy_button_milan_authors = Button(self, text="copy",
                                                command=lambda: self.copy(self.text_milan_authors))
        self.copy_button_milan_authors.grid(row=3, column=2)

        self.text_abstract = Text(self)
        self.text_abstract.config(height=10)
        self.text_abstract.grid(row=4, sticky=N, columnspan=2)
        self.copy_button_milan_authors = Button(self, text="copy",
                                                command=lambda: self.copy(self.text_abstract))
        self.copy_button_milan_authors.grid(row=4, column=2)

        self.text_keywords = Text(self)
        self.text_keywords.config(height=10)
        self.text_keywords.grid(row=4, sticky=N, columnspan=2)
        self.copy_button_milan_authors = Button(self, text="copy",
                                                command=lambda: self.copy(self.text_keywords))
        self.copy_button_milan_authors.grid(row=4, column=2)

    def __init__(self, url, institution, master=None):
        Frame.__init__(self, master)
        self.institution = institution
        self.pack()
        self.createWidgets(url)


def main():
    from optparse import OptionParser
    parser = OptionParser(usage="usage: %prog inspire_url")
    parser.epilog = "example: parse_inspire.py http://inspirehep.net/record/1240088"
    parser.add_option("--institution", type=str, default="Milan U", help="which institution you want to find. Default = 'Milan U'")
    parser.add_option("--no-gui", action="store_true", default=False, help="do no show GUI")
    (options, args) = parser.parse_args()

    if options.no_gui:
        parse(args[0], options.institution)
        exit()

    root = Tk()
    app = Application(args[0] if len(args) else "",
                      institution=options.institution, master=root)
    app.mainloop()
    root.destroy()


if __name__ == '__main__':
    main()


import urllib.request, urllib.error, urllib.parse
import re

r = re.compile("arXiv:([0-9\.]+)")


def get_html(url):
    return urllib.request.urlopen(url).read().decode('utf-8')


def arxiv_number_from_inspire(html_inspire):
    m = r.search(html_inspire)
    if not m:
        raise ValueError("cannot find arxiv from inspire")
    return m.group(1)



#!/usr/bin/env python

__doc__ = "simple tool to dump keyword and abstract from inspirehep website"
__author__ = "Ruggero Turra"
__email__ = "ruggero.turra@mi.infn.it"

import logging
import re

from common import get_html
logging.basicConfig(level=logging.INFO)

#re_author_keys = re.compile('<meta +name="keywords" +content="(.+?)" ?/>')
re_keys = re.compile('<a href="(?:http://inspirehep.net)?/search\?p=keyword.+?">(.+?)</a>', re.DOTALL)
re_other_keys = re.compile('<meta +content="(.+?)" +name="citation_keywords" ?/>')


def get_keywords(html):
    print(type(html))
    m_iter_keys = re_keys.finditer(html)
    if not m_iter_keys:
        logging.error("cannot find author keys")
        exit()
    keys = [key.group(1).strip() for key in m_iter_keys]

    return keys


def format_keys(keys):
    return " ; ".join(keys)


def get_keys_from_html(html):
    keys = get_keywords(html)
    return format_keys(keys)


def main(url):
    logging.info("getting html from %s", url)
    html = get_html(url)
    logging.info("searching from keys")
    keys = get_keywords(html)
    return format_keys(keys)

if __name__ == "__main__":
    from optparse import OptionParser
    parser = OptionParser()
    parser.epilog = "example: python dump_keyword.py  http://inspirehep.net/record/1114314"
    (options, args) = parser.parse_args()

    if len(args) != 1:
        logging.error("you have to specify the arxiv url")
        exit()

    print(main(args[0]))



import logging
logging.basicConfig(level=logging.INFO)
import re

try:
    import xmltodict
except ImportError:
    print """
you have to install xmltodict:

su
pip install xmltodict
"""

from common import get_html
import dump_keyword


def format_name(author):
    name = author[1]
    surname = author[0]
    names = name.split()
    initials = "".join(["%s." % n[0] for n in names])
    return "%s %s" % (initials, surname)


def format_name_italian(author):
    name = author[1]
    surname = author[0]
    names = name.split()
    initials = "".join(["%s." % n[0] for n in names])
    return "%s, %s" % (surname, name)


def main(html_inspire, default_institution):
    m = re.search(r"/([0-9]+)", html_inspire)
    if m is None:
        raise ValueError("not valid html")
    inspire_number = m.group(1)
    url = "http://inspirehep.net/record/{0}/export/xn".format(inspire_number)
    xml = get_html(url)
    doc = xmltodict.parse(xml)

    authors = get_authors(doc)

    print "\n" + "=" * 10 + " ALL AUTHORS " + "=" * 10
    print ", ".join(map(format_name, authors))

    print "\n found %d authors" % len(authors)
    milan_authors = [author for author in authors if (default_institution in " ".join(author[2]))]

    print "\n" + "=" * 10 + (" %s AUTHORS " % default_institution) + "=" * 10
    print "\n".join(map(format_name_italian, milan_authors))

    print "\n" + "=" * 10 + " TITLE " + "=" * 10
    print get_title(doc)

    print "\n" + "=" * 10 + " ABSTRACT " + "=" * 10
    print get_abstract(doc)

    print "\n===== KEYWORKDS ======\n"
    keys = dump_keyword.get_keys_from_html(get_html(html_inspire))
    print keys


def get_abstract(xml_dict):
    return xml_dict['articles']['article']['front']['abstract']


def get_title(xml_dict):
    meta = xml_dict['articles']['article']['front']['article-meta']
    title = meta['title-group']['article-title']
    return title


def get_authors(xml_dict):
    authors = []

    meta = xml_dict['articles']['article']['front']['article-meta']
    title = meta['title-group']['article-title']
    contrib = meta['contrib-group']['contrib']
    for c in contrib:
        author_institutions = []
        aff = c["aff"]
        if len(aff) == 1:
            author_institutions.append(aff["institution"])
        else:
            for a in aff:
                author_institutions.append(a["institution"])

        authors.append((c["name"]["surname"],
                        c["name"]["given-names"],
                        author_institutions))
    return authors


if __name__ == "__main__":
    from optparse import OptionParser
    parser = OptionParser(usage="usage: %prog inspire_url")
    parser.epilog = "example: python dump_milano_authors.py http://inspirehep.net/record/1240088"
    parser.add_option("--institution", type=str, default="Milan U", help="which institution you want to find. Default = 'Milan U'")
    (options, args) = parser.parse_args()

    if len(args) != 1:
        logging.error("you have to specify the inspire url")
        exit()

    main(args[0], options.institution)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="AirStuff",
    version="0.0.2",
    author="Ruggero Turra",
    author_email="ruggero.turra@cern.ch",
    description="Utilities to speedup AIR (Archivio Istituzionale della Ricerca) document upload, for HEP users",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/wiso/airstuff",
    packages=setuptools.find_packages(),
    install_requires=['bibtexparser', 'requests', 'bs4', 'xmltodict'],
    entry_points={'console_scripts': ['parse_inspire=airstuff.parse_inspire:main'], },
    python_requires='>=3.4',
    classifiers=[
        "Programming Language :: Python :: 3",
        "Development Status :: 3 - Alpha",
        "Operating System :: OS Independent",
    ],
)
